@@ -20,12 +20,16 @@ runtest_resort (){
     $EXEPREFIX paexec "$@" 2>&1 | resort
 }
 
+runpaargs (){
+    $EXEPREFIX paargs "$@" 2>&1
+}
+
 cut_version (){
-    awk '$1 == "paexec" {$2 = "x.y.x"} {print}'
+    awk '$1 == "paexec" || $1 == "paargs" {$2 = "x.y.x"} {print}'
 }
 
 cut_help (){
-    awk 'NR <= 3'
+    awk '/^OPTIONS/ {exit 0} {print}'
 }
 
 cut_full_path_closed_stdin (){
@@ -333,23 +337,220 @@ cmp (){
 }
 
 do_test (){
+    runpaargs -V | cut_version |
+cmp 'paargs -V' 'paargs x.y.x written by Aleksey Cheusov
+'
+
+    runpaargs -h 2>&1 | cut_help |
+cmp 'paargs -h' 'paargs -- wrapper for paexec
+usage: paargs [OPTIONS]
+'
+
+    runpaargs -t ssh -P localhost < /dev/null 2>&1 |
+    cmp 'paargs missing free arguments' \
+'paargs: missing arguments. Run paargs -h for details
+'
+
+    printf 'AAA\nBBB\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\nZ Y X\n' |
+	runpaargs -P +3 -I '{}' echo xxx '{}' yyy |
+	paexec_reorder -lgy -Ms |
+    cmp 'paargs -P #1.1' \
+'1 xxx AAA yyy
+1 success
+2 xxx BBB yyy
+2 success
+3 xxx TRTRTR'"'"'BRBRBR yyy
+3 success
+4 xxx CCC yyy
+4 success
+5 xxx DDD yyy
+5 success
+6 xxx EEE yyy
+6 success
+7 xxx FFF yyy
+7 success
+8 xxx "Y;X yyy
+8 success
+9 xxx GGG yyy
+9 success
+10 xxx Z Y X yyy
+10 success
+'
+
+        # -J
+    printf 'aaa\nbbb\nz y x\ntrtrtrbrbrbr\nccc\nddd\neee\nfff\ny;x\nggg\n' |
+	runpaargs -P+2 -I '$$' -c 'awk "BEGIN {print toupper(\"$$\")}"' |
+	paexec_reorder -lgy -Ms |
+    cmp 'paargs -P #1.2' \
+'1 AAA
+1 success
+2 BBB
+2 success
+3 Z Y X
+3 success
+4 TRTRTRBRBRBR
+4 success
+5 CCC
+5 success
+6 DDD
+6 success
+7 EEE
+7 success
+8 FFF
+8 success
+9 Y;X
+9 success
+10 GGG
+10 success
+'
+
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+	runpaargs -P +3 -I '{}' -c 'echo "{} is equal to {}"' |
+	paexec_reorder -lgy -Ms |
+    cmp 'paargs -J #1.3' \
+'1 AAA is equal to AAA
+1 success
+2 BBB is equal to BBB
+2 success
+3 Z Y X is equal to Z Y X
+3 success
+4 TRTRTR'"'"'BRBRBR is equal to TRTRTR'"'"'BRBRBR
+4 success
+5 CCC is equal to CCC
+5 success
+6 DDD is equal to DDD
+6 success
+7 EEE is equal to EEE
+7 success
+8 FFF is equal to FFF
+8 success
+9 "Y;X is equal to "Y;X
+9 success
+10 GGG is equal to GGG
+10 success
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+	runpaargs -I '{}' -c "echo '{} is equal to {}'" -P +3 |
+	paexec_reorder -lgy -Ms |
+    cmp 'paargs -J #1.4' \
+'1 $1 is equal to $1
+1 success
+2 $1 is equal to $1
+2 success
+3 $1 is equal to $1
+3 success
+4 $1 is equal to $1
+4 success
+5 $1 is equal to $1
+5 success
+6 $1 is equal to $1
+6 success
+7 $1 is equal to $1
+7 success
+8 $1 is equal to $1
+8 success
+9 $1 is equal to $1
+9 success
+10 $1 is equal to $1
+10 success
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+	runpaargs -P+3 -I '{}' echo '{}' is equal to '{}' |
+	paexec_reorder -lgy -Ms |
+    cmp 'paargs -J #1.5' \
+'1 AAA is equal to AAA
+1 success
+2 BBB is equal to BBB
+2 success
+3 Z Y X is equal to Z Y X
+3 success
+4 TRTRTR'"'"'BRBRBR is equal to TRTRTR'"'"'BRBRBR
+4 success
+5 CCC is equal to CCC
+5 success
+6 DDD is equal to DDD
+6 success
+7 EEE is equal to EEE
+7 success
+8 FFF is equal to FFF
+8 success
+9 "Y;X is equal to "Y;X
+9 success
+10 GGG is equal to GGG
+10 success
+'
+
+    printf 'aaa\nbbb\nz y x\ntrtrtr'\''brbrbr\nccc\nddd\neee\nfff\n"y;x\nggg\n' |
+	runpaargs -P+3 awk 'BEGIN {print toupper(ARGV [1])}' |
+	paexec_reorder -lgy -Ms |
+    cmp 'paargs #1.1' \
+'1 AAA
+1 success
+2 BBB
+2 success
+3 Z Y X
+3 success
+4 TRTRTR'"'"'BRBRBR
+4 success
+5 CCC
+5 success
+6 DDD
+6 success
+7 EEE
+7 success
+8 FFF
+8 success
+9 "Y;X
+9 success
+10 GGG
+10 success
+'
+
+    # -x + -t
+    export PAEXEC_ENV='   ZZZZ, , ,YYYY,CCCC LALALA    '
+    export ZZZZ=zz1234zz
+    export YYYY=yy1234yy
+    export CCCC=cc1234cc
+    printf 'a\nbb\nccc\ndddd\neeeee\nffffff\n' |
+    runpaargs -t paexec_notransport \
+	-P '1 2 3 4 5 6 7 8 9' \
+	awk 'BEGIN {print ENVIRON["ZZZZ"], ENVIRON["YYYY"], ENVIRON["CCCC"], ENVIRON["LALALA"], toupper(ARGV[1])}' |
+    paexec_reorder -lgy -Ms | grep -v success | resort |
+    cmp 'paargs -t #1 (PAEXEC_ENV)' \
+'1 zz1234zz yy1234yy cc1234cc  A
+2 zz1234zz yy1234yy cc1234cc  BB
+3 zz1234zz yy1234yy cc1234cc  CCC
+4 zz1234zz yy1234yy cc1234cc  DDDD
+5 zz1234zz yy1234yy cc1234cc  EEEEE
+6 zz1234zz yy1234yy cc1234cc  FFFFFF
+'
+    unset ZZZZ YYYY CCCC PAEXEC_ENV
+
+    #
+    runpaargs -t ssh -P localhost -c echo file1 file2 < /dev/null 2>&1 |
+    cmp 'paargs bad files' \
+'paargs: extra arguments. Run paargs -h for details
+'
+
+############################################################    
     runtest -V | cut_version |
 cmp 'paexec -V' 'paexec x.y.x written by Aleksey Cheusov
 '
 
     runtest -h 2>&1 | cut_help |
-cmp 'paexec -h' 'paexec - parallel executor
+cmp 'paexec -h' 'paexec -- parallel executor
          that distributes tasks over CPUs or machines in a network.
 usage: paexec    [OPTIONS]
+       paexec -C [OPTIONS] cmd [args...]
 '
 
     # bad -md= arg
-    runtest -md= 2>&1 |
-    cmp 'paexec -md= #1' 'paexec: bad argument for -md=. Exactly one character is allowed
-'
-
-    runtest -md=aa 2>&1 |
-    cmp 'paexec -md= #2' 'paexec: bad argument for -md=. Exactly one character is allowed
+    runtest -cfake -n+3 -md=aa 2>&1 |
+    cmp 'paexec -md= #1' 'paexec: bad argument for -md=. At most one character is allowed
 '
 
     # bad -ms=/-mf/-mF arg
@@ -398,6 +599,118 @@ usage: paexec    [OPTIONS]
     runtest -t ssh -n localhost -Cx  < /dev/null 2>&1 |
     cmp 'paexec bad -C' \
 'paexec: missing arguments. Run paexec -h for details
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+    runtest -J '{}' -c 'echo "xxx{}yyy"' -n +3 | sort |
+    cmp 'paexec -J #1.1' \
+'xxx"Y;Xyyy
+xxxAAAyyy
+xxxBBByyy
+xxxCCCyyy
+xxxDDDyyy
+xxxEEEyyy
+xxxFFFyyy
+xxxGGGyyy
+xxxTRTRTR'"'"'BRBRBRyyy
+xxxZ Y Xyyy
+'
+
+    # -J
+    printf 'aaa\nbbb\nz y x\ntrtrtrbrbrbr\nccc\nddd\neee\nfff\ny;x\nggg\n' |
+    runtest -n+2 -J '$$' -c 'awk "BEGIN {print toupper(\"$$\")}"' | sort |
+    cmp 'paexec -J #1.2' \
+'AAA
+BBB
+CCC
+DDD
+EEE
+FFF
+GGG
+TRTRTRBRBRBR
+Y;X
+Z Y X
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+    runtest -J '{}' -c 'echo "{} is equal to {}"' -n +3 | sort |
+    cmp 'paexec -J #1.3' \
+'"Y;X is equal to "Y;X
+AAA is equal to AAA
+BBB is equal to BBB
+CCC is equal to CCC
+DDD is equal to DDD
+EEE is equal to EEE
+FFF is equal to FFF
+GGG is equal to GGG
+TRTRTR'"'"'BRBRBR is equal to TRTRTR'"'"'BRBRBR
+Z Y X is equal to Z Y X
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+    runtest -J '{}' -c "echo '{} is equal to {}'" -n +3 | sort |
+    cmp 'paexec -J #1.4' \
+'$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+$1 is equal to $1
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+    runtest  -n +3 -J '{}' -C echo '{}' 'is equal to' '{}' | sort |
+    cmp 'paexec -J #1.5' \
+'"Y;X is equal to "Y;X
+AAA is equal to AAA
+BBB is equal to BBB
+CCC is equal to CCC
+DDD is equal to DDD
+EEE is equal to EEE
+FFF is equal to FFF
+GGG is equal to GGG
+TRTRTR'"'"'BRBRBR is equal to TRTRTR'"'"'BRBRBR
+Z Y X is equal to Z Y X
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+    runtest  -n +3 -J '{}' -C echo '{}' is not equal to ' {}' | sort |
+    cmp 'paexec -J #1.6' \
+'"Y;X is not equal to  $1
+AAA is not equal to  $1
+BBB is not equal to  $1
+CCC is not equal to  $1
+DDD is not equal to  $1
+EEE is not equal to  $1
+FFF is not equal to  $1
+GGG is not equal to  $1
+TRTRTR'"'"'BRBRBR is not equal to  $1
+Z Y X is not equal to  $1
+'
+
+    # -J
+    printf 'AAA\nBBB\nZ Y X\nTRTRTR'\''BRBRBR\nCCC\nDDD\nEEE\nFFF\n"Y;X\nGGG\n' |
+    runtest  -n +3 -J '{}' -C echo '{}/' is not equal to '{}' | sort |
+    cmp 'paexec -J #1.7' \
+'$1/ is not equal to "Y;X
+$1/ is not equal to AAA
+$1/ is not equal to BBB
+$1/ is not equal to CCC
+$1/ is not equal to DDD
+$1/ is not equal to EEE
+$1/ is not equal to FFF
+$1/ is not equal to GGG
+$1/ is not equal to TRTRTR'"'"'BRBRBR
+$1/ is not equal to Z Y X
 '
 
     # x
@@ -571,6 +884,17 @@ fake5.flac
 17 success
 18 failure
 18 /etc/dir with spaces;/etc/dir with spaces/subdir
+'
+
+    find . -type f -name '*.wav' -print0 |
+	runtest -n+6 -0x -C echo |
+    sort | sed 's|^[.]/||' |
+    cmp 'paexec -0' \
+'fakeflac/fake1.wav
+fakeflac/fake2.wav
+fakeflac/fake3.wav
+fakeflac/fake4.wav
+fakeflac/fake5.wav
 '
 
     # toupper
@@ -2168,7 +2492,27 @@ success
     test_tasks1 |
     runtest -W0 -e -d -c cmd_make_package \
 	-n +1 2>&1 | grep '^sum_weight' |
-    cmp 'paexec -W0 #2' \
+    cmp 'paexec -W0 #2.1' \
+'sum_weight [pcc]=4
+sum_weight [gcc]=10
+sum_weight [tcl]=8
+sum_weight [glibc]=9
+sum_weight [python]=7
+sum_weight [dictd]=3
+sum_weight [mplayer]=11
+sum_weight [pike]=6
+sum_weight [ruby]=5
+sum_weight [gnome]=12
+sum_weight [kde]=13
+sum_weight [runawk]=2
+sum_weight [mk-configure]=1
+sum_weight [qt4]=14
+'
+
+    test_tasks1 | sed 's/^weight:/W:/' |
+    runtest -W0 -e -d -mw=W: -c cmd_make_package \
+	-n +1 2>&1 | grep '^sum_weight' |
+    cmp 'paexec -W0 -mw=W: #2.2' \
 'sum_weight [pcc]=4
 sum_weight [gcc]=10
 sum_weight [tcl]=8
